/**
 * 
 */
package org.processmining.plugins.petrinet.analysis;

import javax.swing.JComponent;

import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.connections.ConnectionManager;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.Progress;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.connections.petrinets.behavioral.AggCGGraphConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.ViewSpecificAttributeMap;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.analysis.AggCGGraph;
import org.processmining.models.graphbased.directed.petrinet.analysis.AggTSGraph;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.transitionsystem.AcceptStateSet;
import org.processmining.models.graphbased.directed.transitionsystem.CoverabilityGraph;
import org.processmining.models.graphbased.directed.transitionsystem.StartStateSet;
import org.processmining.models.graphbased.directed.transitionsystem.State;
import org.processmining.models.semantics.Semantics;
import org.processmining.models.semantics.petrinet.Marking;

/**
 * @author Arya Adriansyah
 * @email a.adriansyah@tue.nl
 * @version Jun 10, 2010
 */
@Plugin(name = "Visualize Aggregated Coverability Graph", returnLabels = { "Visualized Coverability Graph" }, returnTypes = { JComponent.class }, parameterLabels = { "Aggregated Coverability Graph" }, userAccessible = false)
@Visualizer
public class AggCGGraphVisualizer {
	private static final float[] INITIAL_DASH_PATTERN = { 7f };

	@PluginVariant(requiredParameterLabels = { 0 })
	public JComponent visualizeWithAll(PluginContext context, AggCGGraph aggCGGraph) {
		try {
			AggCGGraphConnection con = context.getConnectionManager().getFirstConnection(AggCGGraphConnection.class,
					context, aggCGGraph);
			PetrinetGraph n = con.getObjectWithRole(AggCGGraphConnection.NET);
			Semantics<Marking, Transition> semantics = con.getObjectWithRole(AggCGGraphConnection.SEMANTICS);
			return visualizeWithAll(context, n, aggCGGraph, semantics);
		} catch (Exception exc) {
			return null;
		}

	}

	public JComponent visualizeWithAll(PluginContext context, PetrinetGraph n, AggCGGraph aggCGGraph,
			Semantics<Marking, Transition> semantics) {
		// use GraphInformationPanel
		ViewSpecificAttributeMap viewSpecificMap = new ViewSpecificAttributeMap();

		// show marking
		ConnectionManager cm = context.getConnectionManager();
		Marking m = null;
		// Try to see if there is an initial marking available for visualization
		try {
			InitialMarkingConnection mnc = cm.getFirstConnection(InitialMarkingConnection.class, context, n);
			m = mnc.getObjectWithRole(InitialMarkingConnection.MARKING);
			if (m != null) {
				for (Place p : m.baseSet()) {
					String label = "" + m.occurrences(p);
					viewSpecificMap.putViewSpecific(p, AttributeMap.LABEL, label);
					viewSpecificMap.putViewSpecific(p, AttributeMap.TOOLTIP, p.getLabel());
					viewSpecificMap.putViewSpecific(p, AttributeMap.SHOWLABEL, !label.equals(""));
				}
			}
		} catch (ConnectionCannotBeObtained e) {
			// no initial marking, no problem
		}

		return getComponent(n, m, aggCGGraph, viewSpecificMap, context.getProgress(), semantics);
	}

	private JComponent getComponent(PetrinetGraph n, Marking m, AggCGGraph aggCGGraph,
			ViewSpecificAttributeMap viewSpecificMap, Progress progress, Semantics<Marking, Transition> semantics) {
		CoverabilityGraph ts = (CoverabilityGraph) aggCGGraph.get(AggCGGraph.COVERABILITYGRAPH);
		if (aggCGGraph.containsKey(AggTSGraph.STARTSTATESET) || aggCGGraph.containsKey(AggTSGraph.ACCEPTSTATESET)) {
			StartStateSet starts = (StartStateSet) aggCGGraph.get(AggCGGraph.STARTSTATESET);
			AcceptStateSet accepts = (AcceptStateSet) aggCGGraph.get(AggCGGraph.ACCEPTSTATESET);
			for (State state : ts.getNodes()) {
				/**
				 * Note that, in fact, the set of start states is the the set of
				 * start state ids.
				 */
				if (starts.contains(state.getIdentifier())) {
					/**
					 * This state is a start state.
					 */
					state.getAttributeMap().put(AttributeMap.DASHPATTERN, INITIAL_DASH_PATTERN);
				}
				if (accepts.contains(state.getIdentifier())) {
					/**
					 * This state is an accept state.
					 */
					state.setAccepting(true);
				}
			}
		}
		AggTSGraphInformationPanel panel = new AggTSGraphInformationPanel(n, m, progress);
		panel.addCoverabilityGraph(ts, semantics);
		panel.validate();
		return panel;
	}
}
