/**
 * 
 */
package org.processmining.plugins.petrinet.analysis;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;

import javax.swing.JComponent;

import org.jgraph.event.GraphSelectionEvent;
import org.jgraph.event.GraphSelectionListener;
import org.jgraph.graph.GraphSelectionModel;
import org.processmining.framework.plugin.Progress;
import org.processmining.framework.util.Cast;
import org.processmining.framework.util.collection.MultiSet;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.ViewSpecificAttributeMap;
import org.processmining.models.graphbased.directed.jgraph.GraphInformationPanel;
import org.processmining.models.graphbased.directed.jgraph.JGraphVisualizationPanel;
import org.processmining.models.graphbased.directed.jgraph.ProMJGraphVisualizer;
import org.processmining.models.graphbased.directed.jgraph.elements.ProMGraphCell;
import org.processmining.models.graphbased.directed.jgraph.elements.ProMGraphEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.transitionsystem.State;
import org.processmining.models.graphbased.directed.transitionsystem.TransitionSystem;
import org.processmining.models.semantics.Semantics;
import org.processmining.models.semantics.petrinet.CTMarking;
import org.processmining.models.semantics.petrinet.Marking;

/**
 * @author Arya Adriansyah
 * @email a.adriansyah@tue.nl
 * @version Jun 10, 2010
 */
public class AggTSGraphInformationPanel extends GraphInformationPanel {
	private static final long serialVersionUID = 4934501934618950225L;
	private final Marking initialMarking;
	private Collection<PetrinetNode> selectedNodes = Collections.emptySet();

	public AggTSGraphInformationPanel(PetrinetGraph net, Marking initialMarking, Progress progress) {
		super(net, new ViewSpecificAttributeMap(), progress);
		if (initialMarking == null) {
			this.initialMarking = new Marking();
		} else {
			this.initialMarking = initialMarking;
		}
		for (Place p : this.initialMarking.baseSet()) {
			String label = "" + initialMarking.occurrences(p);
			viewSpecificMap.putViewSpecific(p, AttributeMap.LABEL, label);
			viewSpecificMap.putViewSpecific(p, AttributeMap.TOOLTIP, p.getLabel());
			viewSpecificMap.putViewSpecific(p, AttributeMap.SHOWLABEL, !label.equals(""));
		}
	}

	public AggTSGraphInformationPanel(PetrinetGraph net, Progress progress) {
		super(net, new ViewSpecificAttributeMap(), progress);
		initialMarking = new Marking();

	}

	public void addCoverabilityGraph(TransitionSystem ts, Semantics<?, ?> sem) {
		super.addComponent("<html>Coverability Graph<br><I>" + sem.toString() + "</html>", getComponent(ts));
	}

	public void addReachabilityGraph(TransitionSystem ts, Semantics<?, ?> sem) {
		super.addComponent("<html>Reachability Graph<br><I>" + sem.toString() + "</html>", getComponent(ts));
	}

	private JComponent getComponent(TransitionSystem ts) {
		JGraphVisualizationPanel graphPanel = ProMJGraphVisualizer.visualizeGraph(ts);
		GraphSelectionModel model = graphPanel.getGraph().getSelectionModel();
		model.setSelectionMode(GraphSelectionModel.SINGLE_GRAPH_SELECTION);
		model.addGraphSelectionListener(new GraphSelectionListener() {

			public void valueChanged(GraphSelectionEvent evt) {
				for (Object cell : evt.getCells()) {
					if (evt.isAddedCell(cell)) {
						if (cell instanceof ProMGraphCell) {
							ProMGraphCell node = (ProMGraphCell) cell;
							State state = (State) node.getNode();
							Marking m = (Marking) state.getIdentifier();
							selectNodes(m);
						}
						if (cell instanceof ProMGraphEdge) {
							ProMGraphEdge edge = (ProMGraphEdge) cell;
							org.processmining.models.graphbased.directed.transitionsystem.Transition state = (org.processmining.models.graphbased.directed.transitionsystem.Transition) edge
									.getEdge();
							Transition t = (Transition) state.getIdentifier();
							Collection<Transition> set = new ArrayList<Transition>(1);
							set.add(t);
							selectNodes(set);
						}
					}
				}

			}

		});

		return graphPanel;
	}

	private void unselectNodes() {
		for (PetrinetNode p : selectedNodes) {
			viewSpecificMap.clearViewSpecific(p);
		}
		for (Place p : initialMarking.baseSet()) {
			String label = "" + initialMarking.occurrences(p);
			viewSpecificMap.putViewSpecific(p, AttributeMap.LABEL, label);
			viewSpecificMap.putViewSpecific(p, AttributeMap.TOOLTIP, p.getLabel());
			viewSpecificMap.putViewSpecific(p, AttributeMap.SHOWLABEL, !label.equals(""));
		}
	}

	private <T extends PetrinetNode> void selectNodes(Collection<T> newSelection) {
		unselectNodes();

		// iterate through selected invariants
		// Cast is safe, as this is a read-only operation
		if (newSelection instanceof MultiSet<?>) {
			selectedNodes = new HashSet<PetrinetNode>(((MultiSet<T>) newSelection).baseSet());
		} else {
			selectedNodes = new HashSet<PetrinetNode>(newSelection);
		}

		for (PetrinetNode p : selectedNodes) {
			String label;
			label = getNodeLabel(p, newSelection, false);

			viewSpecificMap.putViewSpecific(p, AttributeMap.FILLCOLOR, Color.ORANGE);
			viewSpecificMap.putViewSpecific(p, AttributeMap.LABEL, label);
			viewSpecificMap.putViewSpecific(p, AttributeMap.TOOLTIP, getNodeLabel(p, newSelection, true));
			viewSpecificMap.putViewSpecific(p, AttributeMap.SHOWLABEL, !label.equals(""));
		}

		// refresh the figure
		viewSpecificMapUpdated();
	}

	private String getNodeLabel(PetrinetNode node, Collection<? extends PetrinetNode> set, boolean tooltip) {
		String label = "<html>" + getNodeLabel(node, tooltip);

		if (set.contains(node) && (set instanceof MultiSet<?>)) {
			MultiSet<PetrinetNode> mset = Cast.<MultiSet<PetrinetNode>>cast(set);
			if (tooltip) {
				label += "<br> In Selection: ";
				if (CTMarking.class.isAssignableFrom(mset.getClass())) {
					// we need to show a marking, i.e. ignore the existing label
					label += Cast.<CTMarking>cast(mset).getOmegaPlaces().contains(node) ? "infinite" : ""
							+ mset.occurrences(node);
				} else {
					label += mset.occurrences(node);
				}
			} else {
				if (CTMarking.class.isAssignableFrom(mset.getClass())
						&& Cast.<CTMarking>cast(mset).getOmegaPlaces().contains(node)) {
					// we need to show a marking, i.e. append the existing label
					label += "<br>(&infin;)";
				} else {
					label += "<br>(" + mset.occurrences(node) + ")";
				}
			}
		}
		return label + "</html>";
	}

	private String getNodeLabel(PetrinetNode node, boolean tooltip) {
		String label;
		if (tooltip) {
			label = "Label: ";
		} else {
			label = "";
		}
		label += node.getLabel();
		if (node instanceof Place) {
			if (tooltip) {
				label += "<br> Initial Marking: ";
				label += initialMarking.occurrences(node);
			}

		}
		return label;
	}
}
