/**
 * 
 */
package org.processmining.plugins.petrinet.analysis;

import javax.swing.JComponent;

import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.connections.ConnectionManager;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.Progress;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.connections.petrinets.behavioral.AggPNBehavioralAnalysisResultConnection;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.connections.petrinets.structural.AggPNStructuralAnalysisResultConnection;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.ViewSpecificAttributeMap;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.analysis.AggPNBehavioralAnalysisResult;
import org.processmining.models.graphbased.directed.petrinet.analysis.DeadTransitionsSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.HomeMarkingSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.NetAnalysisInformation;
import org.processmining.models.graphbased.directed.petrinet.analysis.NonLiveTransitionsSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.UnboundedPlacesSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.UnboundedSequences;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.Semantics;
import org.processmining.models.semantics.petrinet.Marking;

/**
 * @author Arya Adriansyah
 * @email a.adriansyah@tue.nl
 * @version Jun 9, 2010
 */
@Plugin(name = "Visualize Petri net Behavioral Analysis", returnLabels = { "Visualized Behavioral Analysis Result" }, returnTypes = { JComponent.class }, parameterLabels = { "Behavioral Analysis Result" }, userAccessible = false)
@Visualizer
public class PNBehavioralAnalysisVisualizer {
	@PluginVariant(requiredParameterLabels = { 0 })
	public JComponent visualizeWithAll(PluginContext context, AggPNBehavioralAnalysisResult aggPNBehAnalysisResult) {
		try {
			AggPNBehavioralAnalysisResultConnection con = context.getConnectionManager().getFirstConnection(
					AggPNBehavioralAnalysisResultConnection.class, context, aggPNBehAnalysisResult);
			PetrinetGraph n = con.getObjectWithRole(AggPNStructuralAnalysisResultConnection.NET);
			return visualizeWithAll(context, n, aggPNBehAnalysisResult, con.getSemantics());
		} catch (Exception exc) {
			return null;
		}

	}

	public JComponent visualizeWithAll(PluginContext context, PetrinetGraph n,
			AggPNBehavioralAnalysisResult aggPNBehAnalysisResult, Semantics<Marking, Transition> semantics) {
		// use GraphInformationPanel
		ViewSpecificAttributeMap viewSpecificMap = new ViewSpecificAttributeMap();

		// show marking
		ConnectionManager cm = context.getConnectionManager();
		Marking m = null;
		// Try to see if there is an initial marking available for visualization
		try {
			InitialMarkingConnection mnc = cm.getFirstConnection(InitialMarkingConnection.class, context, n);
			m = mnc.getObjectWithRole(InitialMarkingConnection.MARKING);
			if (m != null) {
				for (Place p : m.baseSet()) {
					String label = "" + m.occurrences(p);
					viewSpecificMap.putViewSpecific(p, AttributeMap.LABEL, label);
					viewSpecificMap.putViewSpecific(p, AttributeMap.TOOLTIP, p.getLabel());
					viewSpecificMap.putViewSpecific(p, AttributeMap.SHOWLABEL, !label.equals(""));
				}
			}
		} catch (ConnectionCannotBeObtained e) {
			// no initial marking, no problem
		}

		return getComponent(n, m, aggPNBehAnalysisResult, viewSpecificMap, context.getProgress(), semantics);
	}

	private JComponent getComponent(PetrinetGraph n, Marking m, AggPNBehavioralAnalysisResult aggPNBehAnalysisResult,
			ViewSpecificAttributeMap viewSpecificMap, Progress progress, Semantics<Marking, Transition> semantics) {
		PNBehavioralInformationPanel panel = new PNBehavioralInformationPanel(n, m, progress);

		if (aggPNBehAnalysisResult.containsKey(AggPNBehavioralAnalysisResult.BOUNDEDNESS)) {
			Object[] boundednessObj = (Object[]) aggPNBehAnalysisResult.get(AggPNBehavioralAnalysisResult.BOUNDEDNESS);
			panel.addAnalysisInformation("Boundedness Information", (NetAnalysisInformation.BOUNDEDNESS) boundednessObj[0]);
			panel.addNonBoundedPlaces((UnboundedPlacesSet) boundednessObj[1], semantics);
			panel.addUnboundedSequences((UnboundedSequences) boundednessObj[2], semantics);
		}

		if (aggPNBehAnalysisResult.containsKey(AggPNBehavioralAnalysisResult.DEADTRANSITIONS)) {
			panel.addDeadTransitions((DeadTransitionsSet) aggPNBehAnalysisResult
					.get(AggPNBehavioralAnalysisResult.DEADTRANSITIONS), semantics);
		}

		if (aggPNBehAnalysisResult.containsKey(AggPNBehavioralAnalysisResult.HOMEMARKINGSET)) {
			panel.addMarkingSet((HomeMarkingSet) aggPNBehAnalysisResult
					.get(AggPNBehavioralAnalysisResult.HOMEMARKINGSET), semantics);
		}

		if (aggPNBehAnalysisResult.containsKey(AggPNBehavioralAnalysisResult.LIVENESS)) {
			Object[] livenessAnalysisRes = (Object[]) aggPNBehAnalysisResult
					.get(AggPNBehavioralAnalysisResult.LIVENESS);
			panel.addAnalysisInformation("Liveness Information", (NetAnalysisInformation.LIVENESS) livenessAnalysisRes[0]);
			panel.addNonLiveTransitions((NonLiveTransitionsSet) livenessAnalysisRes[1], semantics);
		}

		panel.validate();
		return panel;
	}
}
