/**
 * 
 */
package org.processmining.plugins.petrinet.analysis;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.plugins.petrinet.analysis.PNPluginSelectionPanel.AnalysisTypes;

/**
 * @author Arya Adriansyah
 * @email a.adriansyah@tue.nl
 * @version May 31, 2010
 */
@Plugin(name = "Analyze Structural Property of Petri net", returnLabels = {}, returnTypes = {}, parameterLabels = {
		"Net", "Plugins to be executed" }, userAccessible = true, mostSignificantResult = -1)
public class PNStructuralAnalysis {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "A. Adriansyah", email = "a.adriansyah@tue.nl")
	@PluginVariant(variantLabel = "Analyze Structural Property of Petri Net", requiredParameterLabels = { 0 })
	public void analyzeStructuralProperty(UIPluginContext context, Petrinet net) {
		Set<AnalysisTypes> available = new HashSet<AnalysisTypes>();

		for (AnalysisTypes type : PNPluginSelectionPanel.AnalysisTypes.values()) {
			try {
				if (!type.isBehavioral()) {
					context.getConnectionManager().getFirstConnection(type.getConnectionType(), context, net);
					available.add(type);
				}
			} catch (ConnectionCannotBeObtained e) {
				// Don't care
			}
		}

		Map<AnalysisTypes, Set<AnalysisTypes>> dependencies = DependencyFactory.getStructuralPluginDependencies();

		PNPluginSelectionPanel panel = new PNPluginSelectionPanel(
				available,
				dependencies,
				"Select Analysis Plugins",
				"To select a plugin, click the name of the plugin on the list below. To deselect it, click again the name of the plugin. <br/>"
						+ "If a plug-in depends on other plugin(s), the plugins will automatically be selected.");

		context.showConfiguration("Structural Analysis Plug-in", panel);

		Set<AnalysisTypes> propertiesToBeCalculated = panel.getPropertiesToBeCalculated();

		if (propertiesToBeCalculated.size() > 0) {
			// calculate result
			analyzeStructuralProperty(context, net, propertiesToBeCalculated.toArray(new AnalysisTypes[0]));

		}
	}

	@PluginVariant(variantLabel = "Analyze Structural Property of Petri Net", requiredParameterLabels = { 0, 1 })
	public void analyzeStructuralProperty(PluginContext context, Petrinet net, AnalysisTypes[] pluginToBeExecuted) {

		for (AnalysisTypes type : pluginToBeExecuted) {
			try {
				context.tryToFindOrConstructFirstObject(type.getObjectType(), type.getConnectionType(), type.getRole(),
						net);
			} catch (ConnectionCannotBeObtained e) {
				context.log(e);
			}
		}
	}

}
