/**
 * 
 */
package org.processmining.plugins.petrinet.analysis;

import javax.swing.JComponent;

import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.connections.ConnectionManager;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.Progress;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.connections.petrinets.structural.AggPNStructuralAnalysisResultConnection;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.ViewSpecificAttributeMap;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.analysis.AggPNStructuralAnalysisResult;
import org.processmining.models.graphbased.directed.petrinet.analysis.NetAnalysisInformation;
import org.processmining.models.graphbased.directed.petrinet.analysis.NonExtendedFreeChoiceClustersSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.NonFreeChoiceClustersSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.PTHandles;
import org.processmining.models.graphbased.directed.petrinet.analysis.PlaceInvariantSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.SComponentSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.SiphonSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.TComponentSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.TPHandles;
import org.processmining.models.graphbased.directed.petrinet.analysis.TransitionInvariantSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.TrapSet;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.semantics.petrinet.Marking;

/**
 * @author Arya Adriansyah
 * @email a.adriansyah@tue.nl
 * @version Jun 3, 2010
 */
@Plugin(name = "Visualize Petri net Structural Analysis", returnLabels = { "Visualized Structural Analysis Result" }, returnTypes = { JComponent.class }, parameterLabels = { "Structural Analysis Result" }, userAccessible = false)
@Visualizer
public class PNStructuralAnalysisVisualizer {

	@PluginVariant(requiredParameterLabels = { 0 })
	public JComponent visualizeWithAll(PluginContext context, AggPNStructuralAnalysisResult aggPNStructAnalysisResult) {
		try {
			AggPNStructuralAnalysisResultConnection con = context.getConnectionManager().getFirstConnection(
					AggPNStructuralAnalysisResultConnection.class, context, aggPNStructAnalysisResult);
			PetrinetGraph n = con.getObjectWithRole(AggPNStructuralAnalysisResultConnection.NET);
			return visualizeWithAll(context, n, aggPNStructAnalysisResult);
		} catch (Exception exc) {
			return null;
		}

	}

	public JComponent visualizeWithAll(PluginContext context, PetrinetGraph n,
			AggPNStructuralAnalysisResult aggPNStructAnalysisResult) {
		// use GraphInformationPanel
		ViewSpecificAttributeMap viewSpecificMap = new ViewSpecificAttributeMap();

		// show marking
		ConnectionManager cm = context.getConnectionManager();
		Marking m = null;
		// Try to see if there is an initial marking available for visualization
		try {
			InitialMarkingConnection mnc = cm.getFirstConnection(InitialMarkingConnection.class, context, n);
			m = mnc.getObjectWithRole(InitialMarkingConnection.MARKING);
			if (m != null) {
				for (Place p : m.baseSet()) {
					String label = "" + m.occurrences(p);
					viewSpecificMap.putViewSpecific(p, AttributeMap.LABEL, label);
					viewSpecificMap.putViewSpecific(p, AttributeMap.TOOLTIP, p.getLabel());
					viewSpecificMap.putViewSpecific(p, AttributeMap.SHOWLABEL, !label.equals(""));
				}
			}
		} catch (ConnectionCannotBeObtained e) {
			// no initial marking, no problem
		}

		return getComponent(n, m, aggPNStructAnalysisResult, viewSpecificMap, context.getProgress());
	}

	private JComponent getComponent(PetrinetGraph n, Marking m,
			AggPNStructuralAnalysisResult aggPNStructAnalysisResult, ViewSpecificAttributeMap viewSpecificMap,
			Progress progress) {
		PNStructuralInformationPanel panel = new PNStructuralInformationPanel(n, m, progress);

		if (aggPNStructAnalysisResult.containsKey(AggPNStructuralAnalysisResult.PLACEINVARIANT)) {
			panel.addInvariant((PlaceInvariantSet) aggPNStructAnalysisResult
					.get(AggPNStructuralAnalysisResult.PLACEINVARIANT));
		}

		if (aggPNStructAnalysisResult.containsKey(AggPNStructuralAnalysisResult.TRANSITIONINVARIANT)) {
			panel.addInvariant((TransitionInvariantSet) aggPNStructAnalysisResult
					.get(AggPNStructuralAnalysisResult.TRANSITIONINVARIANT));
		}
		if (aggPNStructAnalysisResult.containsKey(AggPNStructuralAnalysisResult.SIPHONSET)) {
			panel.addComponents((SiphonSet) aggPNStructAnalysisResult.get(AggPNStructuralAnalysisResult.SIPHONSET));
		}
		if (aggPNStructAnalysisResult.containsKey(AggPNStructuralAnalysisResult.TRAPSET)) {
			panel.addComponents((TrapSet) aggPNStructAnalysisResult.get(AggPNStructuralAnalysisResult.TRAPSET));
		}
		if (aggPNStructAnalysisResult.containsKey(AggPNStructuralAnalysisResult.FREECHOICE)) {
			Object[] analysisResult = (Object[]) aggPNStructAnalysisResult
					.get(AggPNStructuralAnalysisResult.FREECHOICE);
			panel.addAnalysisInformation("Free-choice property", (NetAnalysisInformation.FREECHOICE) analysisResult[0]);
			panel.addAnalysisInformation("Extended free-choice property", (NetAnalysisInformation.EXTFREECHOICE) analysisResult[1]);
			panel.addNonFreeChoiceClusters((NonFreeChoiceClustersSet) analysisResult[2]);
			panel.addNonExtendedFreeChoiceClusters((NonExtendedFreeChoiceClustersSet) analysisResult[3]);
		}
		if (aggPNStructAnalysisResult.containsKey(AggPNStructuralAnalysisResult.PTHANDLES)) {
			panel.addNodePairSet((PTHandles) aggPNStructAnalysisResult.get(AggPNStructuralAnalysisResult.PTHANDLES));
		}
		if (aggPNStructAnalysisResult.containsKey(AggPNStructuralAnalysisResult.TPHANDLES)) {
			panel.addNodePairSet((TPHandles) aggPNStructAnalysisResult.get(AggPNStructuralAnalysisResult.TPHANDLES));
		}
		if (aggPNStructAnalysisResult.containsKey(AggPNStructuralAnalysisResult.SCOMPONENTSET)) {
			panel.addComponents((SComponentSet) aggPNStructAnalysisResult
					.get(AggPNStructuralAnalysisResult.SCOMPONENTSET));
		}
		if (aggPNStructAnalysisResult.containsKey(AggPNStructuralAnalysisResult.TCOMPONENTSET)) {
			panel.addComponents((TComponentSet) aggPNStructAnalysisResult
					.get(AggPNStructuralAnalysisResult.TCOMPONENTSET));
		}

		panel.validate();
		return panel;
	}
}
