/**
 * 
 */
package org.processmining.plugins.petrinet.analysis;

import java.awt.Color;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

import javax.swing.DefaultListModel;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListModel;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.processmining.framework.plugin.Progress;
import org.processmining.framework.util.Cast;
import org.processmining.framework.util.HTMLToString;
import org.processmining.framework.util.collection.ComparablePair;
import org.processmining.framework.util.collection.MultiSet;
import org.processmining.models.graphbased.AttributeMap;
import org.processmining.models.graphbased.ViewSpecificAttributeMap;
import org.processmining.models.graphbased.directed.jgraph.GraphInformationPanel;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.analysis.AbstractComponentSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.AbstractInvariantSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.AbstractMarkingSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.AbstractNodePairSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.NetAnalysisInformation;
import org.processmining.models.graphbased.directed.petrinet.analysis.NonExtendedFreeChoiceClustersSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.NonFreeChoiceClustersSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.NotPCoveredNodesSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.NotSCoveredNodesSet;
import org.processmining.models.graphbased.directed.petrinet.analysis.UnconnectedNodesSet;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.semantics.Semantics;
import org.processmining.models.semantics.petrinet.CTMarking;
import org.processmining.models.semantics.petrinet.Marking;

/**
 * @author Arya Adriansyah
 * @email a.adriansyah@tue.nl
 * @version Jun 3, 2010
 */
public class PNStructuralInformationPanel extends GraphInformationPanel {
	private static final long serialVersionUID = -7897646921225620468L;
	private Marking initialMarking = null;

	public PNStructuralInformationPanel(PetrinetGraph net, Marking initialMarking, Progress progress) {
		super(net, new ViewSpecificAttributeMap(), progress);
		if (initialMarking == null) {
			this.initialMarking = new Marking();
		} else {
			this.initialMarking = initialMarking;
		}
		for (Place p : this.initialMarking.baseSet()) {
			String label = "" + initialMarking.occurrences(p);
			viewSpecificMap.putViewSpecific(p, AttributeMap.LABEL, label);
			viewSpecificMap.putViewSpecific(p, AttributeMap.TOOLTIP, p.getLabel());
			viewSpecificMap.putViewSpecific(p, AttributeMap.SHOWLABEL, !label.equals(""));
		}
	}

	public PNStructuralInformationPanel(PetrinetGraph net, Progress progress) {
		super(net, new ViewSpecificAttributeMap(), progress);
		initialMarking = new Marking();
	}

	public void addAnalysisInformation(String label, NetAnalysisInformation<?>... information) {
		if (information.length > 0) {
			super.addComponent(label, new JScrollPane(getAnalysisComponent(information)));
		}
	}

	public void addMarkingSet(AbstractMarkingSet<?> set, Semantics<?, ?> semantics) {
		super.addComponent("<html>" + set.getLabel() + "<br><I>" + semantics.toString() + "</html>", new JScrollPane(
				getMarkingComponent(set.getMarkings())));
	}

	public void addNodePairSet(AbstractNodePairSet<?, ?> set) {
		super.addComponent(set.getLabel(), new JScrollPane(getNodePairComponent(set)));
	}

	public void addInvariant(AbstractInvariantSet<?> marking) {
		super.addComponent("<html>" + marking.getLabel() + "</html>", new JScrollPane(getInvariantComponent(marking)));
	}

	public void addComponents(AbstractComponentSet<?> marking) {
		super
				.addComponent("<html>" + marking.getLabel() + "</html>", new JScrollPane(
						getNodeMarkingComponent(marking)));
	}

	public void addNotSCoveredNodes(NotSCoveredNodesSet component) {
		super.addComponent("<html>Non-S-covered Nodes</html>", new JScrollPane(getNodeMarkingComponent(component)));
	}

	public void addNotPCoveredNodes(NotPCoveredNodesSet component) {
		super.addComponent("<html>Non-P-covered Places</html>", new JScrollPane(getNodeMarkingComponent(component)));
	}

	public void addNonFreeChoiceClusters(NonFreeChoiceClustersSet component) {
		super.addComponent("<html>Non-FC Clusters</html>", new JScrollPane(getNodeMarkingComponent(component)));
	}

	public void addNonExtendedFreeChoiceClusters(NonExtendedFreeChoiceClustersSet component) {
		super.addComponent("<html>Non-XFC Clusters</html>", new JScrollPane(getNodeMarkingComponent(component)));
	}

	public void addUnconnectedNodes(UnconnectedNodesSet component) {
		super.addComponent("<html>Unconnected Nodes</html>", new JScrollPane(getNodeMarkingComponent(component)));
	}

	private Collection<PetrinetNode> selectedNodes = Collections.emptySet();

	private JComponent getAnalysisComponent(NetAnalysisInformation<?>... netAnalysisInformations) {

		//form vector of array of string
		Vector<Object> columnNames = new Vector<Object>();
		columnNames.add("Property");
		columnNames.add("Value");

		Vector<Vector<Object>> data = new Vector<Vector<Object>>();
		for (NetAnalysisInformation<?> netAnalysisInformation : netAnalysisInformations) {
			String key = netAnalysisInformation.getLabel();

			Vector<Object> datum = new Vector<Object>();
			datum.add(key);
			datum.add(netAnalysisInformation.getValue());
			data.add(datum);
		}

		// add the list, set it to non editable
		JTable table = new JTable(data, columnNames) {
			private static final long serialVersionUID = -7116637591741809191L;

			public boolean isCellEditable(int row, int column) {
				return false;
			}
		};

		return new JScrollPane(table);
	}

	private JComponent getMarkingComponent(Marking... markings) {

		// instantiate a model which will store all of invariant value
		DefaultListModel listModel = new DefaultListModel();

		for (Marking m : markings) {
			// add the element to the list
			listModel.addElement(m);
		}

		return new JScrollPane(makeJList(listModel));
	}

	private <T extends PetrinetNode> JComponent getInvariantComponent(AbstractInvariantSet<T> invariantMarking) {

		// instantiate a model which will store all of invariant value
		DefaultListModel listModel = new DefaultListModel();

		Iterator<MultiSet<T>> iterator = invariantMarking.iterator();
		while (iterator.hasNext()) {
			// add the element to the list
			MultiSet<T> invariants = iterator.next();
			listModel.addElement(invariants);
		}
		return new JScrollPane(makeJList(listModel));
	}

	private JComponent getNodePairComponent(AbstractNodePairSet<? extends PetrinetNode, ? extends PetrinetNode> set) {

		// instantiate a model which will store all of invariant value
		DefaultListModel listModel = new DefaultListModel();

		for (ComparablePair<? extends PetrinetNode, ? extends PetrinetNode> pair : set) {
			// add the element to the list
			Collection<PetrinetNode> list = new ArrayList<PetrinetNode>(2);
			list.add(pair.getFirst());
			list.add(pair.getSecond());
			listModel.addElement(list);
		}

		return new JScrollPane(makeJList(listModel));
	}

	private <T extends PetrinetNode> JComponent getNodeMarkingComponent(Set<?> nodeMarking) {

		// instantiate a model which will store all of invariant value
		DefaultListModel listModel = new DefaultListModel();

		Iterator<?> iterator = nodeMarking.iterator();
		while (iterator.hasNext()) {
			// add the element to the list
			Object invariants = iterator.next();
			listModel.addElement(invariants);
		}

		return new JScrollPane(makeJList(listModel));
	}

	private void unselectNodes() {
		for (PetrinetNode p : selectedNodes) {
			viewSpecificMap.clearViewSpecific(p);
		}
		for (Place p : initialMarking.baseSet()) {
			String label = "" + initialMarking.occurrences(p);
			viewSpecificMap.putViewSpecific(p, AttributeMap.LABEL, label);
			viewSpecificMap.putViewSpecific(p, AttributeMap.TOOLTIP, p.getLabel());
			viewSpecificMap.putViewSpecific(p, AttributeMap.SHOWLABEL, !label.equals(""));
		}
	}

	private <T extends PetrinetNode> void selectNodes(Collection<T> newSelection) {
		unselectNodes();

		// iterate through selected invariants
		// Cast is safe, as this is a read-only operation
		if (newSelection instanceof MultiSet<?>) {
			selectedNodes = new HashSet<PetrinetNode>(((MultiSet<T>) newSelection).baseSet());
		} else {
			selectedNodes = new HashSet<PetrinetNode>(newSelection);
		}

		for (PetrinetNode p : selectedNodes) {
			String label;
			label = getNodeLabel(p, newSelection, false);

			viewSpecificMap.putViewSpecific(p, AttributeMap.FILLCOLOR, Color.ORANGE);
			viewSpecificMap.putViewSpecific(p, AttributeMap.LABEL, label);
			viewSpecificMap.putViewSpecific(p, AttributeMap.TOOLTIP, getNodeLabel(p, newSelection, true));
			viewSpecificMap.putViewSpecific(p, AttributeMap.SHOWLABEL, !label.equals(""));
		}

		// refresh the figure
		viewSpecificMapUpdated();
	}

	private JList makeJList(ListModel listModel) {
		final JList markingList = new JList(listModel);
		markingList.setCellRenderer(new HTMLToString.HTMLCellRenderer());

		markingList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		markingList.addListSelectionListener(new ListSelectionListener() {

			@SuppressWarnings("unchecked")
			public void valueChanged(ListSelectionEvent e) {
				if (!e.getValueIsAdjusting()) {
					Collection<? extends PetrinetNode> marking = (Collection<? extends PetrinetNode>) markingList
							.getModel().getElementAt(markingList.getSelectedIndex());

					selectNodes(marking);

				}
			}
		});

		markingList.addMouseListener(new MouseListener() {
			@SuppressWarnings("unchecked")
			public void mouseReleased(MouseEvent e) {
				if (markingList.getSelectedIndex() >= 0) {
					Collection<? extends PetrinetNode> marking = (Collection<? extends PetrinetNode>) markingList
							.getModel().getElementAt(markingList.getSelectedIndex());

					selectNodes(marking);
				}
			}

			public void mousePressed(MouseEvent e) {
			}

			public void mouseExited(MouseEvent e) {
			}

			public void mouseEntered(MouseEvent e) {
			}

			public void mouseClicked(MouseEvent e) {
			}
		});

		return markingList;
	}

	private String getNodeLabel(PetrinetNode node, Collection<? extends PetrinetNode> set, boolean tooltip) {
		String label = "<html>" + getNodeLabel(node, tooltip);

		if (set.contains(node) && (set instanceof MultiSet<?>)) {
			MultiSet<PetrinetNode> mset = Cast.<MultiSet<PetrinetNode>>cast(set);
			if (tooltip) {
				label += "<br> In Selection: ";
				if (CTMarking.class.isAssignableFrom(mset.getClass())) {
					// we need to show a marking, i.e. ignore the existing label
					label += Cast.<CTMarking>cast(mset).getOmegaPlaces().contains(node) ? "infinite" : ""
							+ mset.occurrences(node);
				} else {
					label += mset.occurrences(node);
				}
			} else {
				if (CTMarking.class.isAssignableFrom(mset.getClass())
						&& Cast.<CTMarking>cast(mset).getOmegaPlaces().contains(node)) {
					// we need to show a marking, i.e. append the existing label
					label += "<br>(&infin;)";
				} else {
					label += "<br>(" + mset.occurrences(node) + ")";
				}
			}
		}
		return label + "</html>";
	}

	private String getNodeLabel(PetrinetNode node, boolean tooltip) {
		String label;
		if (tooltip) {
			label = "Label: ";
		} else {
			label = "";
		}
		label += node.getLabel();
		if (node instanceof Place) {
			if (tooltip) {
				label += "<br> Initial Marking: ";
				label += initialMarking.occurrences(node);
			}

		}
		return label;
	}
}
